<?php
/**
 * @author MahaCOD Programming Team
 * @copyright www.mahacod.ir
 * 
 */
class MahaCOD_Carrier extends CarrierModule{
    public function __construct(){
        $this->name = 'mahacod_carrier';
        $this->tab = 'shipping_logistics';
        $this->version = '1.0';
		$this->author = 'MahaCOD';
        $this->limited_countries = array('ir','us'); // iran
        parent::__construct();
        $this->displayName = $this->l('Mahacod Carrier Service');
        $this->description = $this->l('Mahacod Webservice');
        $this->confirmUninstall = $this->l('Are you sure to want uninstall this module');
    }
	/*********************************************************************************/
    public function install(){
		$mahacod_tracking_url = 'http://mahacod.ir/order_tracking.html?orderid=@';
        $carrierConfig = array(0 => array(
										'uname' => 'pishtaz-cod',
										'name' => $this->l('mahacod-pishtaz-cod'),
										'url' => $mahacod_tracking_url,
										'id_tax_rules_group' => 0,
										'active' => true,
										'deleted' => 0,
										'shipping_handling' => false,
										'range_behavior' => 0,
										'delay' => array('en' => 'Description 1'
														, Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => $this->l('high price speed send')),
										'id_zone' => 3,
										'is_module' => true,
										'shipping_external' => true,
										'external_module_name' => $this->name,
										'need_range' => true)
								, 1 => array(
										'uname' => 'sefareshi-cod',
										'name' => $this->l('mahacod-sefareshi-cod'),
										'url' => $mahacod_tracking_url,
										'id_tax_rules_group' => 0,
										'active' => true,
										'deleted' => 0,
										'shipping_handling' => false,
										'range_behavior' => 0,
										'delay' => array('en' => 'Description 1'
														, Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => $this->l('low price normal send')),
										'id_zone' => 3,
										'is_module' => true,
										'shipping_external' => true,
										'external_module_name' => $this->name,
										'need_range' => true)
								);
        $id_pishtaz_cod = $this->installExternalCarrier($carrierConfig[0]);
        $id_sefareshi_cod = $this->installExternalCarrier($carrierConfig[1]);
        Configuration::updateValue('MAHACOD_PISHTAZ_C_ID', (int)$id_pishtaz_cod);
        Configuration::updateValue('MAHACOD_SEFARESHI_C_ID', (int)$id_sefareshi_cod);

        if (!parent::install() 
			or !$this->registerHook('updateCarrier') 
			or !$this->registerHook('cart') 
			or !$this->registerHook('newOrder') 
			or !$this->registerHook('DisplayBeforeCarrier')){
            return false;
		}
        return true;
    }
	/*********************************************************************************/
    public function uninstall(){
        // Uninstall
        if (!parent::uninstall() 
			or !$this->unregisterHook('updateCarrier') 
			or !$this->unregisterHook('cart') 
			or !$this->unregisterHook('newOrder') 
			or !$this->unregisterHook('header')){
            return false;
		}
        // Delete External Carrier
        $pishtaz_cod = new Carrier((int)(Configuration::get('MAHACOD_PISHTAZ_C_ID')));
        $sefareshi_cod = new Carrier((int)(Configuration::get('MAHACOD_SEFARESHI_C_ID')));
        $pishtaz_cod = new Carrier((int)(Configuration::get('MAHACOD_PISHTAZ_C_ID')));
        $sefareshi_cod = new Carrier((int)(Configuration::get('MAHACOD_SEFARESHI_C_ID')));
		@unlink(_PS_SHIP_IMG_DIR_.(int)$pishtaz_cod->id.'.jpg');
		@unlink(_PS_SHIP_IMG_DIR_.(int)$sefareshi_cod->id.'.jpg');
		
        // If external carrier is default set other one as default
        if (Configuration::get('PS_CARRIER_DEFAULT') == (int)($pishtaz_cod->id) ||
            Configuration::get('PS_CARRIER_DEFAULT') == (int)($sefareshi_cod->id)){
            global $cookie;
            $carriersD = Carrier::getCarriers($cookie->id_lang, true, false, false, null,PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);
            foreach ($carriersD as $carrierD){
                if ($carrierD['active'] and !$carrierD['deleted'] and ($carrierD['name'] != $this->_config['name'])){
                    Configuration::updateValue('PS_CARRIER_DEFAULT', $carrierD['id_carrier']);
				}
			}
        }
        // delete Carrier
        $pishtaz_cod->deleted = 1;
        $sefareshi_cod->deleted = 1;
        if (!$pishtaz_cod->update() || !$sefareshi_cod->update()){
            return false;
		}
        return true;
    }
	/*********************************************************************************/
    public function installExternalCarrier($config){
        $carrier = new Carrier();
        $carrier->name = $config['name'];
        $carrier->url = $config['url'];
        $carrier->id_tax_rules_group = $config['id_tax_rules_group'];
        $carrier->id_zone = $config['id_zone'];
        $carrier->active = $config['active'];
        $carrier->deleted = $config['deleted'];
        $carrier->delay = $config['delay'];
        $carrier->shipping_handling = $config['shipping_handling'];
        $carrier->range_behavior = $config['range_behavior'];
        $carrier->is_module = $config['is_module'];
        $carrier->shipping_external = $config['shipping_external'];
        $carrier->external_module_name = $config['external_module_name'];
        $carrier->need_range = $config['need_range'];
        $languages = Language::getLanguages(true);
        foreach ($languages as $language){
            if ($language['iso_code'] == 'en'){
                $carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
			}
            if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT'))){
                $carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
			}
        }
        if ($carrier->add()){
            $groups = Group::getGroups(true);
            foreach ($groups as $group){
                Db::getInstance()->autoExecute(_DB_PREFIX_ . 'carrier_group', array('id_carrier' =>(int)($carrier->id), 'id_group' => (int)($group['id_group'])), 'INSERT');
			}
            // price range
            $rangePrice = new RangePrice();
            $rangePrice->id_carrier = $carrier->id;
            $rangePrice->delimiter1 = '0';
            $rangePrice->delimiter2 = '10000000';
            $rangePrice->add();
            // weight range
            $rangeWeight = new RangeWeight();
            $rangeWeight->id_carrier = $carrier->id;
            $rangeWeight->delimiter1 = '0';
            $rangeWeight->delimiter2 = '100000';
            $rangeWeight->add();
            $zones = Zone::getZones(true);
            foreach ($zones as $zone){
				Db::getInstance()->autoExecute(_DB_PREFIX_ . 'carrier_zone', array('id_carrier' =>(int)($carrier->id), 'id_zone' => (int)($zone['id_zone'])), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_ . 'delivery', array(
					'id_carrier' => (int)($carrier->id),
					'id_range_price' => (int)($rangePrice->id),
					'id_range_weight' => null,
					'id_zone' => (int)($zone['id_zone']),
					'price' => '0'), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_ . 'delivery', array(
					'id_carrier' => (int)($carrier->id),
					'id_range_price' => null,
					'id_range_weight' => (int)($rangeWeight->id),
					'id_zone' => (int)($zone['id_zone']),
					'price' => '0'), 'INSERT');
            }
			if (!copy(dirname(__FILE__).'/views/img/'.$config['uname'].'.jpg', _PS_SHIP_IMG_DIR_.(int)$carrier->id.'.jpg')){
            	return false;
			}

            return (int)($carrier->id);
        }
        return false;
    }
	/*********************************************************************************/
    public function hookUpdateCarrier($params){
        if ((int)($params['id_carrier']) == (int)(Configuration::get('MAHACOD_PISHTAZ_C_ID'))){
            Configuration::updateValue('MAHACOD_PISHTAZ_C_ID', (int)($params['carrier']->id));
		}
        if ((int)($params['id_carrier']) == (int)(Configuration::get('MAHACOD_SEFARESHI_C_ID'))){
            Configuration::updateValue('MAHACOD_SEFARESHI_C_ID', (int)($params['carrier']->id));
		}
    }
	/*********************************************************************************/
    public function getOrderShippingCost($params, $shipping_cost){
        $newCookie=new Cookie('mahacod');
        $carrier = false;
        if (Tools::getValue('step') == 2){
            $id = $params->id_carrier;
            if (Configuration::get('MAHACOD_PISHTAZ_C_ID') == $id){
                $carrier = $newCookie->__get('pishtaz_carrier');
			}elseif (Configuration::get('MAHACOD_SEFARESHI_C_ID') == $id){
                $carrier = $newCookie->__get('sefareshi_carrier');
			}
        }elseif(Tools::getValue('step') == 3){
            $id = $params->id_carrier;
            if (Configuration::get('MAHACOD_PISHTAZ_C_ID') == $id){
                $carrier = $newCookie->__get('pishtaz_carrier');
			}elseif (Configuration::get('MAHACOD_SEFARESHI_C_ID') == $id){
                $carrier = $newCookie->__get('sefareshi_carrier');
			}
            $newCookie->__set('mahacodCarrier', $carrier);
            #$this->context->cookie->__unset('sefareshi_carrier');
            #$this->context->cookie->__unset('pishtaz_carrier');
        }
        $iso_rial = Currency::getIdByIsoCode('IRR');
        $rial = new Currency($iso_rial);
        $current_currency = new Currency($this->context->cookie->id_currency);
        if ($carrier !=0){
            if ($current_currency->id == $rial->id){
                return $carrier;
			}else{
                return Tools::convertPriceFull($carrier, $rial, $current_currency);
			}
        }elseif ($newCookie->__isset('mahacodCarrier')){
            if ($current_currency->id == $rial->id){
                return $newCookie->__get('mahacodCarrier');
			}else{
                return Tools::convertPriceFull($newCookie->__get('mahacodCarrier'), $rial,$current_currency);
			}
        }else{
            return 0;
		}
    }
	/*********************************************************************************/
    public function getOrderShippingCostExternal($params){
        return $this->getOrderShippingCost($params,0);
    }
	/*********************************************************************************/
    public function hookDisplayBeforeCarrier($params){
        $newCookie = new CookieCore('mahacod');
		$method = (!$newCookie->__get('payment_method')) ? 0 : $newCookie->__get('payment_method');
        $id_sefareshi = Configuration::get('MAHACOD_PISHTAZ_C_ID');
        $id_pishtaz = Configuration::get('MAHACOD_SEFARESHI_C_ID');
        $this->context->controller->addJS(_MODULE_DIR_.$this->name.'/views/js/jquery.blockUI.js');
        $this->smarty->assign(array(
            'mahacod_carrier_rul' => $method,
            'ajaxurl' => _MODULE_DIR_.'mahacod/views/ajax/ajaxprocess.php?action=getCarrierPrice',
            'module' => $this->name,
            'mahacod_pcid'=>$id_sefareshi,
            'mahacod_scid'=>$id_pishtaz,
           // 'mahacod_online'=>$this->getCarriers($id_sefareshi,$id_pishtaz),
            ));
        return $this->display(__FILE__, 'views/templates/front/carrierlistjs.tpl');
     }
	/*********************************************************************************/
    public function getCarriers($id_sefareshi,$id_pishtaz){
        $ids = array();
        $query = new DbQuery();
        $query->select('id_carrier');
        $query->from('carrier');
        $query->where('active=1');
        $results = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query->build());
        if(!$results){
            return false;
		}
        foreach($results as $result){
            if($id_sefareshi != $result['id_carrier'] and $id_pishtaz != $result['id_carrier']){
				$ids[] = $result['id_carrier'];
			}
        }
        return $ids;
    }
}
